/* ReSCAL - Global definitions
 *
 * Copyright (C) 2011
 *
 * Author: Olivier Rozier <rozier@ipgp.fr>
 *
 * This file is part of ReSCAL.
 *
 * ReSCAL is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



//numero et date de version
#define VER_NUM "1.0"
#define VER_DAT "2011-05"

// ======================================================

// choix du modele

//#define MODELE_CMB  //Core-Mantle Boundary
//#define MODELE_ICB  //InnerCore-Core Boundary
//#define MODELE_RIV  //Riviere (sans air)
//#define MODELE_CHE  //Chenal (avec air)
//#define MODELE_CRI  //Cristallisation (de calcite)
#define MODELE_DUN    //Saltation et deposition
//#define MODELE_AVA    //Avalanche (de grains)
//#define MODELE_DIF  //Diffusion

// ======================================================

// options communes
#define CYCLAGE_HOR //cyclage horizontal (incompatible avec TRANSITION_BORD)
#define DUMP_IMAGE //generation d'images (necessite la presence de la librairie libgd)
#define INFO_DBL //fichier info sur les listes de cellules et doublets actifs
#define INFO_CEL //fichier info sur le nombre de cellules de chaque type
#define INFO_TRANS //fichier info sur le nombre de transitions et de correlations effectives
//#define PARALLEL  //plusieurs process en parallele
#define DUMP_SIGNATURE //pour verifier si deux simulations sont vraiment identiques
//#define LOG_FILE //generation des logs dans un fichier texte
// ===================== MODELE CMB =====================
#ifdef MODELE_CMB

// nom du modele
#define MOD_NAME "CMB"

// list des etats
#define PLUS  0 //solide
#define ZERO  1 //liquide sature en elements legers
#define MOINS 2 //liquide non-sature
#define BORD  3 //bord
#define DUM   4 //inerte
#define PIERRE 7 //granit
#define TUNNEL 9 //tunnel vers une terre parallele

// noms des etats
#define ETATS { "PLUS", "ZERO", "MOINS", "BORD", "DUM", "", "", "GRANIT", "", "TUNNEL" }

// options specifiques du modele
//#define TRANSITION_BORD
#define DUMP_RUGOSI
#endif

// ===================== MODELE DUNES =====================
#ifdef MODELE_DUN

// nom du modele
#define MOD_NAME "DUN"

// list des etats
#define GR    0 //grain stable
#define GRJ   1 //grain instable jeune
#define GRV   2 //grain instable vieux
#define EAUC  3 //eau
#define EAUT  4 //turbulence
#define BORD  5 //bord
#define DUM   6 //inerte
#define IN    7 //source de grains
#define OUT   8 //sortie de grains
#define TUNNEL 9 //tunnel vers une terre parallele

// noms des etats
#define ETATS { "GR", "GRJ", "GRV", "EAUC", "EAUT", "BORD", "DUM", "IN", "OUT", "TUNNEL" }

// options specifiques du modele
#define ALTI GR //calcul de la hauteur des piles de cellules GR
//#define AVALANCHES    //activation des anciennes avalanches (parametre Ava_delai)
//#define PROPAGATION_AVA //avalanches synchrones instantanees localisees (avec propagation)
#define TRANSITION_AVA //activation des transitions d'avalanche (parametre Lambda_A), incompatible avec l'option AVALANCHES
#define AVA_NORM //utilisation des normales dans les avalanches (parametre Ava_angle)
#define ROTATIONS
#ifndef CYCLAGE_HOR
#define TRANSITION_BORD
#endif
#define LIENS_TRANSITIONS
//#define DUMP_RUGOSI
//#define TRACE_TRANS
//#define TRACE3D_CEL
#define COLLISIONS //automate gaz sur reseau
#define CGV //controle par le gradient de vitesse (avec COLLISIONS)
//#define APPEL_MCC //Matlab application
//#define CELL_TIME GR //datation des cellules GR
//#define CELL_COLOR //cellules colorees
//#define NORM2D //CGV avec normales en 2d
//#define DUMP_SIGMA //calcul sigma sur l'altitude (avec le peigne)
//#define DUMP_AUTOCORREL //calcul de l'auto-correlation sur l'altitude (avec le peigne)
#endif

// ===================== MODELE AVA =====================
#ifdef MODELE_AVA

// nom du modele
#define MOD_NAME "AVA"

// list des etats
#define GR    0 //grain stable
#define AIR   1 //air
#define BORD  2 //bord
#define DUM   3 //inerte
#define IN    4 //source de grains
#define OUT   5 //sortie de grains
#define TUNNEL 9 //tunnel vers une terre parallele

#define TERRE    GR

// noms des etats
#define ETATS { "GR", "AIR", "BORD", "DUM", "IN", "OUT", "", "", "", "TUNNEL" }

// options specifiques du modele
#define ALTI GR //calcul de la hauteur des piles de cellules GR
//#define AVALANCHES  //avalanches synchrones instantanees globales
//#define PROPAGATION_AVA //avalanches synchrones instantanees localisees (avec propagation)
#define TRANSITION_AVA //activation des transitions d'avalanche (parametre Lambda_A), incompatible avec l'option AVALANCHES
#define AVA_NORM //utilisation des normales dans les avalanches (parametre Ava_angle)
//#define ROTATIONS //table tournante
//#define TRANSITION_BORD
//#define CELL_TIME GR //datation des cellules GR
#define CELL_COLOR //cellules colorees
//#define NORM2D //pour l'affichage
//#define TRACE_TRANS
#endif

// ===================== MODELE ICB =====================
#ifdef MODELE_ICB

// nom du modele
#define MOD_NAME "ICB"

// list des etats
#define PLUS  0 //fer solide
#define ZERO  1 //liquide sature en elements legers
#define MOINS 2 //liquide non-sature
#define BORD  3 //bord
#define TUNNEL 9 //tunnel vers une terre parallele

// noms des etats
#define ETATS { "PLUS", "ZERO", "MOINS", "BORD" }

// options specifiques du modele
#define DUMP_RUGOSI

#endif

// ===================== MODELE RIV =====================
#ifdef MODELE_RIV

// nom du modele
#define MOD_NAME "RIVIERE"

// list des etats
#define TERRE  0 //terre
#define BOUE   1 //boue
#define EAU    2 //eau
#define BT     3 //boue turbulent
#define PIERRE 4 //pierre
#define BORD   5 //bord
#define DUM    6 //inerte
#define IN     7 //source
#define OUT    8 //sortie
#define TUNNEL 9 //tunnel vers une terre parallele

// noms des etats
#define ETATS { "TERRE", "BOUE", "EAU", "BT", "PIERRE", "BORD", "DUM", "IN", "OUT", "TUNNEL"}

// options specifiques du modele
#define TRANSITION_BORD
#define LIENS_TRANSITIONS
#define ALTI TERRE //calcul de la hauteur des piles de cellules TERRE
#define AVALANCHES
//#define SURRECTIONS
//#define ETAT_BT //boue qui tombe -> cones de deposition
//#define REGUL_DB //regulation des transitions de doublet en fonction de l'altitude
//#define TRACE_SRC //calcul bassin versant
//#define TRACE_AIRE  //calcul aires drainees
//#define TRACE_TRANS
//#define TRACE_PAR //calcul debit + pente
//#define APPEL_GMT
//#define LISSAGE  //couche uniforme de BOUE + cristallisation BOUE->TERRE + pas de transport vers le haut => lissage avant mode TRACE
//#define CELL_TIME TERRE //datation des cellules TERRE


#endif

// ===================== MODELE CHE =====================
#ifdef MODELE_CHE

// nom du modele
#define MOD_NAME "CHENAL"

// list des etats
#define TERRE 0 //terre
#define BOUE  1 //boue
#define EAU   2 //eau
#define BORD  3 //bord
#define DUM   4 //inerte
#define IN    5 //source d'eau
#define AIR   6 //air
#define PIERRE 7 //pierre
#define TUNNEL 9 //tunnel vers une terre parallele

// noms des etats
#define ETATS { "TERRE", "BOUE", "EAU", "BORD", "BIDON", "SOURCE", "AIR", "PIERRE" }

// options specifiques du modele
#define TRANSITION_BORD
//#define DIFFUSION_TERRE_EAU
#define LIENS_TRANSITIONS

#endif

// ===================== MODELE CRI =====================
#ifdef MODELE_CRI

// nom du modele
#define MOD_NAME "CRISTALLISATION"

// list des etats
#define PLUS  0 //solide
#define ZERO  1 //gaz
#define BORD  3 //bord
#define IN    5 //source du gaz
#define AIR   6 //air
#define TUNNEL 9 //tunnel vers une terre parallele

// noms des etats
#define ETATS { "SOLIDE", "GAZ", "", "BORD", "", "SOURCE", "AIR" }

// options specifiques du modele
#define TRANSITION_BORD

#endif

// ===================== MODELE DIF =====================
#ifdef MODELE_DIF

// nom du modele
#define MOD_NAME "DIFFUSION"

// list des etats
#define TERRE 0 //terre
#define EAU  1 //eau
#define AIR   2 //air
#define BORD  3 //bord
#define DUM   4 //inerte
#define IN    5 //source
#define BOUE  7 //boue

// noms des etats
#define ETATS { "TERRE", "EAU", "AIR", "BORD", "BIDON", "SOURCE", "", "BOUE" }

// options specifiques du modele
#define TRANSITION_BORD

#endif


// ======================================================

// constantes communes

#define MAX_CELL  10
#define MAX_DB    100

#define DROITE    0
#define BAS       1
#define DERRIERE  2

#define GAUCHE    3
#define HAUT      4
#define DEVANT    5

#define VERTICAL   0
#define EST_OUEST  1
#define NORD_SUD   2
#define HORIZONTAL 3 // EST_OUEST ou NORD_SUD
#define ISOTROPE   4
#define CLASSES_DB { "VERTICAL", "EST_OUEST", "NORD_SUD", "HORIZONTAL" ,"ISOTROPE"}

#define MAX_TRANSITIONS_DB 100
#define MAX_TRANSITIONS_CEL 10
#define MAX_LIENS  100

#define MAX_TRANSITIONS MAX_TRANSITIONS_DB+MAX_TRANSITIONS_CEL

#define SIMUL_DEBUT 0
#define SIMUL_INIT 0
#define SIMUL_EN_COURS 2
#define SIMUL_PAUSE 3
#define SIMUL_STOP  4
#define SIMUL_FIN   5

#ifdef PARALLEL
#define MAX_NODE  16
#endif

#define PI 3.14159

// ======================================================

