/* ReSCAL - States transitions
 *
 * Copyright (C) 2011
 *
 * Author: Olivier Rozier <rozier@ipgp.fr>
 *
 * This file is part of ReSCAL.
 *
 * ReSCAL is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



enum {TR_NOTYPE, TR_TRANSPORT, TR_INOUT} trans_types;

typedef int Callback_regul(void *data);
typedef int Callback_check(int ix, void *data);

typedef struct une_transition_de_doublet{
  char reference;
  unsigned char classe; //horizontale, verticale ou isotrope
  unsigned char depart, arrivee;  //doublets de depart et d'arrivee
  double intensite;
  char type;  //type de transition
  char LHF, RDB; //indicateur "cellules modifiees : left/high/front - right/down/behind"
  char LTR; //indicateur "lien de transitions"
  Callback_regul *regul; //fonction de regulation globale de l'intensite
  Callback_check *check;  //fonction de controle local d'une transition (inhibition si elle retourne 0)
  char chk_cel; //indique si le controle s'effectue sur la premiere ou la deuxieme cellule du doublet
} TransitionDb;

typedef struct une_transition_de_cellule{
  char reference;
  unsigned char depart, arrivee;
  double intensite;
  Callback_regul *regul; //fonction de regulation globale de l'intensite
} TransitionCel;

typedef struct un_lien_entre_deux_transitions_de_doublets{
  int trans1, trans2;
  char cel; //indique si le lien s'effectue sur la premiere ou la deuxieme cellule du doublet
  double intensite;
} LienTransDb;

typedef struct data_check{
  //int trans;   //transition
  char dir; //direction
  char cel; //premiere ou deuxieme cellule
} DataCheck;

void init_transitions();
void trans_ref(char ref, char classe, char cel_depart_1, char cel_depart_2, char cel_arrivee_1, char cel_arrivee_2, double intensite);
void trans(char classe, char cel_depart_1, char cel_depart_2, char cel_arrivee_1, char cel_arrivee_2, double intensite);
void trans_type(char ref, char type);
void trans_regul(char ref, Callback_regul f);
void trans_check(char ref, Callback_check chk, char cel);
void lien_trans(char ref1, char ref2, char ltr_cel, double intensite);
int split_trans_hor(int db_hor);
void trans_cel_ref(char ref, char cel_depart, char cel_arrivee, double intensite);
void trans_cel(char cel_depart, char cel_arrivee, double intensite);
void trans_cel_regul(char ref, Callback_regul f);
void do_trans_db(int tr, int ix, int dir);
void do_trans_cel(int tr, int ix);
void dump_transitions();
void dump_trans_info();
void dump_ln_info();

